# -*- coding: utf-8 -*-

CHUNK_SIZE = 4096

class FileField(object):
    def __init__(self, filename):
        self.filename = filename
        
    def chunks(self, size=CHUNK_SIZE):
        raise NotImplemented()
    
    def data(self):
        return ''.join(self.chunks())
    
    def save(self, path):
        with open(path, 'wb') as f:
            for chunk in self.chunks():
                f.write(chunk)
    


# -*- coding: utf-8 -*-
from decorated.base.function import Function
from metaweb.errors import ValidationError
import doctest
import re

class Validator(Function):
    def _call(self, *args, **kw):
        value = self._evaluate_expression(self._param, *args, **kw)
        error = self._validate(value)
        if error is None:
            return super(Validator, self)._call(*args, **kw)
        else:
            raise ValidationError(self._code, param=self._param, message=error)
        
    def _init(self, param, code):
        super(Validator, self)._init()
        self._param = param
        self._code = code
        
    def _validate(self, value):
        raise NotImplementedError()

class Type(Validator):
    def _init(self, param, type_or_types, code='BAD_ARGUMENT_TYPE'):
        super(Type, self)._init(param, code)
        self._type_or_types = type_or_types
        
    def _validate(self, value):
        if not isinstance(value, self._type_or_types):
            return 'Value should be of type %s.' % (self._type_or_types,)
type = Type

class LengthRange(Type):
    def _init(self, param, lower, upper, code='BAD_ARGUMENT_LENGTH'):
        super(LengthRange, self)._init(param, basestring, code=code)
        self._lower = lower
        self._upper = upper
        
    def _validate(self, value):
        error = super(LengthRange, self)._validate(value)
        if error is not None:
            return error
        length = len(value)
        if length < self._lower or length > self._upper:
            return 'String length should be within [%s, %s].' % (self._lower, self._upper)
length_range = LengthRange

class Range(Type):
    def _init(self, param, lower, upper, code='ARGUMENT_OUT_OF_RANGE'):
        super(Range, self)._init(param, (int, float), code=code)
        self._lower = lower
        self._upper = upper
        
    def _validate(self, value):
        error = super(Range, self)._validate(value)
        if error is not None:
            return error
        if value < self._lower or value > self._upper:
            return 'Value should be within [%s, %s].' % (self._lower, self._upper)
range = Range

class Required(Validator):
    def _init(self, param, code='ARGUMENT_MISSING'):
        super(Required, self)._init(param, code)
        
    def _validate(self, value):
        if value is None or value == '':
            return 'Argument cannot be null.'
required = Required
    
class Regex(Type):
    def _init(self, param, pattern, code='BAD_ARGUMENT'):
        super(Regex, self)._init(param, code, basestring)
        self._pattern = re.compile(pattern)
        
    def _validate(self, value):
        if not self._pattern.match(value):
            return 'Value does not match "%s".' % self._pattern.pattern
regex = Regex

if __name__ == '__main__':
    doctest.testmod()
    


# -*- coding: utf-8 -*-
import doctest

class Path(str):
    def __init__(self, path):
        '''
        >>> path = Path('/test')
        >>> path
        '/test'
        >>> path.args
        {}
        '''
        super(Path, self).__init__(path)
        self.args = {}
        self.view = None
    
if __name__ == '__main__':
    doctest.testmod()
    


# -*- coding: utf-8 -*-
import doctest

class WebError(Exception):
    def __init__(self, status, code, message='', **extras):
        self.status = status
        self.code = code
        self.message = message
        self._extras = extras
        
    def __json__(self):
        '''
        >>> e = WebError(500, 'INTERNAL_ERROR', 'Error message.')
        >>> e.__json__()
        {'message': 'Error message.', 'code': 'INTERNAL_ERROR'}
        
        >>> e = WebError(500, 'INTERNAL_ERROR', 'Error message.', param='key')
        >>> e.__json__()
        {'message': 'Error message.', 'code': 'INTERNAL_ERROR', 'param': 'key'}
        '''
        result = {
            'code': self.code,
            'message': self.message,
        }
        result.update(self._extras)
        return result
        
    def __str__(self):
        '''
        >>> e = WebError(500, 'INTERNAL_ERROR', 'Error message.')
        >>> str(e)
        '[INTERNAL_ERROR] Error message.'
        '''
        return '[%s] %s' % (self.code, self.message)
    
class ValidationError(WebError):
    def __init__(self, code, param=None, message=''):
        super(ValidationError, self).__init__(400, code, message=message, param=param)
        
    def __str__(self):
        '''
        >>> str(ValidationError('INVALID_ARGUMENT', param='key', message='Error message.'))
        '[INVALID_ARGUMENT] Error message. (param=key)'
        >>> str(ValidationError('INVALID_ARGUMENT', message='Error message.'))
        '[INVALID_ARGUMENT] Error message.'
        '''
        result = super(ValidationError, self).__str__()
        param = self._extras.get('param')
        if param is not None:
            result += ' (param=%s)' % param
        return result
    
if __name__ == '__main__':
    doctest.testmod()
    


# -*- coding: utf-8 -*-
from fixtures2.case import TestCase
from fixtures2.patches import PatchesFixture

class TestCase(TestCase):
    def setUp(self):
        super(TestCase, self).setUp()
        self.patches = self.useFixture(PatchesFixture())
        self.patches.patch('metaweb.views._pending_views', [])
        self.patches.patch('metaweb.views._abs_pathes', {})
        self.patches.patch('metaweb.views._regex_pathes', {})
